const mongoose = require('mongoose');
const mongoosePaginate = require('mongoose-paginate-v2');
const slugify = require('slugify');  

const productSchema = new mongoose.Schema({
  title: {
    type: String,
    required: true,
  },
  content: {
    type: String,
  },
  photos:{
    type: [String]
  },
  price:{
    type: Number
  },
  size:{
    type: Number
  },
  color:{
    type: String
  },
  quality:{
    type: String
  },
  status:{
    type: String
  },
  method:{
    type: String
  },
  user_id: {
    type: mongoose.Schema.Types.ObjectId,
    ref: 'User'
  },
  slug:{
    type: String,
    unique: true
  }

}, {
  timestamps: true,
});

productSchema.plugin(mongoosePaginate);

productSchema.pre('save', async function(next) {
  try {
    if (!this.isModified('title')) {
      return next();
    }
    this.slug = slugify(this.title, { lower: true, strict: true });
   
    // Check for uniqueness
    const slugRegEx = new RegExp(`^(${this.slug})((-[0-9]*$)?)$`, 'i');
    const postsWithSlug = await this.constructor.find({ slug: slugRegEx });
   
    if (postsWithSlug.length) {
      this.slug = `${this.slug}-${postsWithSlug.length + 1}`;
    }
    next();
  } catch (error) {
    next(error);  // Pass any errors to the next middleware
  }
 });
 

module.exports = mongoose.model('Product', productSchema);